package v3d.editor;

import java.awt.*;
import java.awt.image.*;

class TGAEncoder implements ImageObserver {
	int width;
	int height;
	int scaledWidth;
	int scaledHeight;
	int downScale;
	Image img;
	int imageData[];
	byte buffer[];
	boolean createImageDone;

	TGAEncoder(Image image,int scale) {
		width = image.getWidth(null);
		height = image.getHeight(null);
		if (width <= 0) width = scale;
		if (height <= 0) height = scale;
		scaledWidth = width/scale;
		scaledHeight = height/scale;
		downScale = scale;
		imageData = new int[width * height];
		buffer = new byte[scaledWidth*scaledHeight*3+18];
		img = image;
	}

	public void imageToData() {
		// grab the pixels
		PixelGrabber pgcol = new PixelGrabber(img, 0, 0, width, height, imageData, 0, width);

		try
		{
		        pgcol.grabPixels( );
		}
		catch( InterruptedException e ) {
			// fill with black if failed
			for (int y=0; y<height; y++) {
				for (int x=0; x<width; x++) {
					imageData[y*width + x] = 0;
				}
			}
		}
	}

	public byte[] getTGA() {
		imageToData();

		// construct TGA header
		int i=0;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x02;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = 0x00;
		buffer[i++] = (byte)(scaledWidth & 0xff);
		buffer[i++] = (byte)((scaledWidth >> 8) & 0xff);
		buffer[i++] = (byte)(scaledHeight & 0xff);
		buffer[i++] = (byte)((scaledHeight >> 8) & 0xff);
		buffer[i++] = 0x18;
		buffer[i++] = 0x00;

		int division = downScale*downScale;
		for (int y=scaledHeight-1; y>=0; y--) {
			for (int x=0; x<scaledWidth; x++) {
				int r = 0;
				int g = 0;
				int b = 0;
				for (int yp=0; yp<downScale; yp++) {
					int ys = (downScale*y + yp)*width;
					for (int xp=0; xp<downScale; xp++) {
 						int xs = downScale*x + xp;
						int col = imageData[ys + xs];
						r += ((col >> 16) & 0xff);
						g += ((col >> 8) & 0xff);
						b += (col & 0xff);
					}
				}
				buffer[i++] = (byte)(b/division);
				buffer[i++] = (byte)(g/division);
				buffer[i++] = (byte)(r/division);
			}
		}

		return buffer;
	}

	public boolean imageUpdate(Image img, int  infoflags, int  x, int  y, int  width, int  height)
	{
		if ((infoflags & ImageObserver.ALLBITS) != 0)
		{
			createImageDone = true;
		}
		return true;
	}
}
